//	GGVR_GPU_Definitions.h
//
//	© 2024 by Jeff Weeks
//	See TermsOfUse.txt

#ifndef GGVR_GPU_Definitions_h
#define GGVR_GPU_Definitions_h

#include <simd/simd.h>

//	As of November 2023, Swift doesn't know about "simd_half3" or "simd_half4".
//	So for now let's create them manually.  Surprisingly enough,
//	this simple definition works!  It even produces 8-byte alignment
//	for "simd_half3", as required for consistency with the 8-byte alignment
//	of the Metal Shading Language's "half3" data type.

typedef __attribute__((__ext_vector_type__(2))) __fp16 faux_simd_half2;
typedef __attribute__((__ext_vector_type__(3))) __fp16 faux_simd_half3;
typedef __attribute__((__ext_vector_type__(4))) __fp16 faux_simd_half4;

typedef struct
{
	//	In Curved-Space Pool Hall,
	//		the "game space" is the table space.
	//
	//	In Curved Spaces VR,
	//		the "game space" is the space of the central fundamental cell,
	//		which is centered at the point where the player's head
	//		was when they opened the manifold.

	//	For unmodified curved-space rendering, it would be enough
	//	to know the gameSpaceToEyeSpace matrix for each eye.
	//	But if we want to apply the azimuthalEquidistantMap()
	//	or compressSouthernHemisphere() we need to know
	//	the gameSpaceToBridgeOfNoseSpace and bridgeOfNoseSpaceToClipSpace
	//	matrices separately.
	
	simd_float4x4	gameSpaceToBridgeOfNoseSpace;		//	the "view matrix"

	//	bridgeOfNoseSpaceToClipSpace maps the bridge-of-nose space
	//	to eye space and then applies the appropriate projection matrices.
	simd_float4x4	bridgeOfNoseSpaceToClipSpace[2];	//	the "projection matrices"

} GGVRViewProjection;

#endif /* GGVR_GPU_Definitions_h */
